pub enum RawError {
UnexpectedType,
MalformedValue(String),
Utf8EncodingError(Vec<u8>),
}Expand description
Error to indicate that either a value was empty or it contained an unexpected type, for use with the direct getters.
Variants
UnexpectedType
Found a Bson value with the specified key, but not with the expected type
MalformedValue(String)
The found value was not well-formed
Utf8EncodingError(Vec<u8>)
Found a value where a utf-8 string was expected, but it was not valid utf-8. The error value contains the malformed data as a string.